#include <iostream>
using namespace std;

#include "Individual.h"
#include "DomesticationException.h"

Individual::Individual(int ploidy, int haploidNumber)
{
  //cout << "Made an individual" << endl;
  _ploidy = ploidy;
  _haplnum = haploidNumber;
  Chromosome **chrmptr;
  
  for (int i = 0; i < _haplnum; i++)
  {
    chrmptr = new Chromosome*[_ploidy];
    for (int j = 0; j < _ploidy; j++)  chrmptr[j] = NULL;
    _genome.push_back(chrmptr);
  }
}

Individual::Individual(const Individual &other)
{
  //cout << "Copying an individual" << endl;
  _ploidy = other.getPloidy();
  _haplnum = other.getHaploidNumber();
  Chromosome **chrmptr;
  
  for (int i = 0; i < _haplnum; i++)
  {
    chrmptr = new Chromosome*[_ploidy];
    for (int j = 0; j < _ploidy; j++)
      chrmptr[j] = new Chromosome(*(other.getGenotype(i, j)));
    _genome.push_back(chrmptr);
  }
}

void Individual::setGenotype(int chrmnum, const vector<const Chromosome*> &chromosomes)
{
  if (chromosomes.size() != _ploidy)
    throw new DomesticationException("Invalid number of chromosomes in vector.");
  
  for (int i = 0; i < _ploidy; i++)
    setGenotype(chrmnum, i, chromosomes.at(i));
}

void Individual::setGenotype(int chrmnum, int ploidy, const Chromosome* chromosome)
{
  if (chrmnum < 0 || chrmnum >= _haplnum)
    throw new DomesticationException("Invalid chromosome number.");
  
  else if (ploidy < 0 || ploidy >= _ploidy)
    throw new DomesticationException("Invalid chromosome copy number.");
  
  Chromosome **chrmptr = _genome.at(chrmnum);

  if (chrmptr[ploidy])  delete chrmptr[ploidy];
  chrmptr[ploidy] = new Chromosome(*chromosome);
}

int Individual::getPloidy() const
{
  return _ploidy;
}

int Individual::getHaploidNumber() const
{
  return _haplnum;
}

const Chromosome * Individual::getGenotype(int chrmnum, int ploidy) const
{
  if (chrmnum < 0 || chrmnum >= _haplnum)
    throw new DomesticationException("Invalid chromosome number.");
  
  else if (ploidy < 0 || ploidy >= _ploidy)
    throw new DomesticationException("Invalid chromosome copy number.");
  
  return _genome.at(chrmnum)[ploidy];
}

Individual::~Individual()
{
  //cout << "deleted an individual" << endl;
  for (vector<Chromosome**>::iterator cait = _genome.begin(); cait != _genome.end(); cait++)
  {
    for (int i = 0; i < _ploidy; i++)
      delete (*cait)[i];
    delete[] *cait;  
  }
}

